/***************************************************************************
 *   Copyright (C) 2004 by Adam Skowronski                                 *
 *   tytus@oldfield.org.pl                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "carddata.h"
#include "wpast.h"
#include "wfuture.h"

#include <iostream.h>

/*!
    \fn CalendarCardData::CalendarCardData()
	Default constructor<br>
	Constructor only sets state property to CalEnums::NO_PIXMAP value
 */
 CalendarCardData::CalendarCardData()
{
	state=CalEnums::NO_PIXMAP;
}


/*!
    \fn CalendarCardData::~CalendarCardData()
	Default destructor. I does nothing.
 */
CalendarCardData::~CalendarCardData()
{
	pWidget=NULL;
}




/*!
    \fn CalendarCardData::getIndividualEventText()
	Retrieves text data from individual event
	\return text of event or empty string if event doesn't exist
 */
QString CalendarCardData::getIndividualEventText()
{
	if(IndividualEvent)
		  return IndividualEvent->text;
 	else
		return "";
}


/*!
    \fn CalendarCardData::getPeriodicEventText()
	Retrieves text data from periodic events
	\return Concatenated string of all periodic events
 */
QString CalendarCardData::getPeriodicEventText()
{
    QString temp="";
	for (uint i = 0; i<PeriodicEvents.count(); i++)
	{
		temp.append(PeriodicEvents.at(i)->text);
		temp.append("\n");
	}
    return temp;
}


/*!
    \fn CalendarCardData::setIndividualEvent(QString tekst, bool bIsPast)
	This method: <br>
	&nbsp;&nbsp;&nbsp;&nbsp;Sets or delete individual event for current CalendarCardData object. <br>
	&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;If <i>tekst</i> is an empty string - removes existent event from the object. If it is not empty - adds or modify individual event.<br>
	&nbsp;&nbsp;&nbsp;&nbsp;Calls methods from CalendarWidget class to add or remove this event from a global list of events as well.<br>
	&nbsp;&nbsp;&nbsp;&nbsp;Sets state property to proper value.
	\param tekst text of individual event to be set
	\param bIsPast indicates if it is a past card
	\return state of an object. \sa state
 */
int CalendarCardData::setIndividualEvent(QString tekst, bool bIsPast)
{
	//check if event exists:
	if (this->IndividualEvent)
		{
		//event exists before
		if (tekst.isEmpty())
		//event is empty - will be deleted
			{
			if(bIsPast)
				{
				((CalendarWidgetPast*)this->parent)->removeIndividualEvent((CalendarEventPast*)this->IndividualEvent);
				((QWidget*)pWidget)->setDisabled(TRUE);
				}
			else
				((CalendarWidgetFuture*)this->parent)->removeIndividualEvent(this->IndividualEvent);
	
			this->IndividualEvent=NULL;
			if (!PeriodicEvents.count())
				this->state=CalEnums::NO_PIXMAP;
			else
				state=evaluateState();
			}
		else
			//update data
			{
			this->IndividualEvent->text=tekst;
			this->state=CalEnums::PXI;
			if (bIsPast)
				((CalendarWidget*)this->parent)->parent->savePastEvents();
			else
				((CalendarWidget*)this->parent)->parent->saveIndividualEvents();
			
			}
		}
	else if (!tekst.isEmpty() && !bIsPast)
	//event doesn't exist, create new one
	{
		this->IndividualEvent=((CalendarWidgetFuture*)this->parent)->addIndividualEvent(date, tekst);
		this->state=CalEnums::PXI;
		if(PeriodicEvents.count())
			this->state=evaluateState();
	}
	else if(tekst.isEmpty() && !PeriodicEvents.count())
			this->state=CalEnums::NO_PIXMAP;
	
	return state;
}


/*!
    \fn CalendarCardData::addPeriodicEvent(CalendarEventPeriodic *event)
	This method:<br>
	- Adds new periodic event to the current CalendarCardData object<br>
	- Sets <i>state</i> property to proper value.
	\param event pointer to existent event
	\return state of an object. \sa state
	\note this method can only add existent event, It doesn't create new event or remove existent one.
	
 */
int CalendarCardData::addPeriodicEvent(CalendarEventPeriodic *event) 
{
	this->PeriodicEvents.append(event);
	this->state=evaluateState();
	if (pWidget)
		{
		this->pWidget->updateState(state);
		}
	return state;
}

/*!
    \fn CalendarCardData::addIndividualEvent(CalendarEvent *event)
	This method:<br>
	- Adds new individual event to the current CalendarCardData object<br>
	- Sets <i>state</i> property to proper value.
	\param event pointer to event
	\return state of an object. \sa state
	\note this method can only add existent event, It doesn't create new event or remove existent one.
 */
int CalendarCardData::addIndividualEvent(CalendarEvent *event)
{
	this->IndividualEvent=event;
	state=CalEnums::PXI;
	return state;
}




/*!
    \fn CalendarCardData::CalendarCardData(QDate d, QWidget* parent)
	Constructor of CalendarCardData object used in application.
	<br>It sets some properties to default values (ie. date).
	\param d date to set \c date property.
	\param parent pointer to parent window \sa CalendarWidget
 */
CalendarCardData::CalendarCardData(QDate d, QWidget* parent)
{
	IndividualEvent=NULL;
	pWidget=NULL;

	this->parent = parent;

	date=d;	
	state=CalEnums::NO_PIXMAP;
	if (d.day()==1)
		this->isFirstOfMonth=TRUE;
	else
		this->isFirstOfMonth=FALSE;
}


/*!
    \fn CalendarCardData::resetCard()
	This method resets the CalendarCardData object into initial state.
	<br>Used during reloading of all events from files.	
	\note This method doesn't remove any event from a global list of events is CalendarWidget class object.
	\return state of an object. \sa state
 */
 int CalendarCardData::resetCard()
{
	state=CalEnums::NO_PIXMAP;
	this->PeriodicEvents.clear();
	this->IndividualEvent->text="";
	return state;
}

/*!
    \fn CalendarCardData::getDate()
	Standard getter.
	<br>This method gets value of \c date property.
	\return \c date property of object
 */
QDate CalendarCardData::getDate()
{
	return this->date;
}



/*!
    \fn CalendarCardData::removePeriodicEvents()
	This methods removes all periodic events from current object of CalendarCardData class. <br>
	\note This method doesn't remove any event from a global list of events is CalendarWidget class object.
	\return state of an object \sa state
 */
int CalendarCardData::removePeriodicEvents()
{
	this->PeriodicEvents.clear();
    if(IndividualEvent)
		state=CalEnums::PXI;
	else
		state=CalEnums::NO_PIXMAP;
	return state;
}


/*!
    \fn CalendarCardData::evaluateState()
	This method evaluate state of an object. 
	<br>It decides which event of object is most important and state is evaluated according to this event.
	\return state of an object. \sa state
 */
CalEnums::PIXMAP_STATES CalendarCardData::evaluateState()
{
	CalEnums::PIXMAP_STATES state=CalEnums::NO_PIXMAP;
	CalEnums::PIXMAP_STATES newstate=state;
	CalendarEventPeriodic * temp;
	if(IndividualEvent)
		return CalEnums::PXI;
	for (int i=0; i< (signed long)(PeriodicEvents.count()); i++)
	{
		temp=PeriodicEvents.at(i);
		switch(temp->eventType)
		{
		case CalEnums::DAILY:
		case CalEnums::PERIOD:
			newstate=CalEnums::PXD;
			break;
		case CalEnums::DAYS:
		case CalEnums::WEEKLY:
			newstate=CalEnums::PXW;
			break;
		case CalEnums::MONTHLY:
			newstate=CalEnums::PXM;
			break;
		case CalEnums::YEARLY:
			newstate=CalEnums::PXY;
			break;
		}
		state=(state>newstate) ? state : newstate;
	}
	return state;
}
